@@ -22,12 +22,9 @@ along with GCC; see the file COPYING3.  If not see
 #include "coretypes.h"
 #include "tm.h"
 #include "tree.h"
-#include "rtl.h"
 #include "tm_p.h"
-#include "hard-reg-set.h"
 #include "basic-block.h"
 #include "output.h"
-#include "diagnostic.h"
 #include "tree-flow.h"
 #include "tree-dump.h"
 #include "timevar.h"
@@ -88,6 +85,26 @@ tree_ssa_unswitch_loops (void)
   /* Go through inner loops (only original ones).  */
   FOR_EACH_LOOP (li, loop, LI_ONLY_INNERMOST)
     {
+      if (dump_file && (dump_flags & TDF_DETAILS))
+        fprintf (dump_file, ";; Considering loop %d\n", loop->num);
+
+      /* Do not unswitch in cold regions. */
+      if (optimize_loop_for_size_p (loop))
+        {
+          if (dump_file && (dump_flags & TDF_DETAILS))
+            fprintf (dump_file, ";; Not unswitching cold loops\n");
+          continue;
+        }
+
+      /* The loop should not be too large, to limit code growth. */
+      if (tree_num_loop_insns (loop, &eni_size_weights)
+          > (unsigned) PARAM_VALUE (PARAM_MAX_UNSWITCH_INSNS))
+        {
+          if (dump_file && (dump_flags & TDF_DETAILS))
+            fprintf (dump_file, ";; Not unswitching, loop too big\n");
+          continue;
+        }
+
       changed |= tree_unswitch_single_loop (loop, 0);
     }
 
@@ -181,31 +198,6 @@ tree_unswitch_single_loop (struct loop *loop, int num)
   gimple stmt;
   bool changed = false;
 
-  /* Only unswitch innermost loops.  */
-  if (loop->inner)
-    {
-      if (dump_file && (dump_flags & TDF_DETAILS))
-	fprintf (dump_file, ";; Not unswitching, not innermost loop\n");
-      return false;
-    }
-
-  /* Do not unswitch in cold regions.  */
-  if (optimize_loop_for_size_p (loop))
-    {
-      if (dump_file && (dump_flags & TDF_DETAILS))
-	fprintf (dump_file, ";; Not unswitching cold loops\n");
-      return false;
-    }
-
-  /* The loop should not be too large, to limit code growth.  */
-  if (tree_num_loop_insns (loop, &eni_size_weights)
-      > (unsigned) PARAM_VALUE (PARAM_MAX_UNSWITCH_INSNS))
-    {
-      if (dump_file && (dump_flags & TDF_DETAILS))
-	fprintf (dump_file, ";; Not unswitching, loop too big\n");
-      return false;
-    }
-
   i = 0;
   bbs = get_loop_body (loop);
   found = loop->num_nodes;
